﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using IASA.Demo.BusinessModel;
using IASA.Demo.Data;
using IASA.Demo.Data.Generic;
using IASA.Demo.CreditCardProvider;
using Microsoft.Practices.Unity;

namespace IASA.Demo.BusinessService
{
    public class OrderService : IASA.Demo.BusinessService.IOrderService
    {
        
        public OrderPurchasedResult ProcessOrder(Customer customer)
        {
            //Calculate Order Total/w product Discount
            if (customer.CurentOrder.IsOrderValid)
            {
                double orderTotal = customer.CalculateCurrentOrder();

                // Add Sales Tax
                ISalesTax salesTaxStrategy = customer.ShipToAddress.SalesTaxStrategy;
                orderTotal += salesTaxStrategy.CalculateSalesTax();

                // Use DI to get credit card provider
                ICreditCardProvider creditProvider = CreditCardProvider.Current.GetCreditCardProvider();
                bool creditProcessed = creditProvider.ProcessTransaction(customer, orderTotal);

                // Check if credit card transaction was succesful
                if (creditProcessed)
                {
                    // Save Order using the Repository
                    OrderRepository repository = new OrderRepository();
                    bool saveResult = repository.Save(customer.CurentOrder);
                }
                else
                {
                    orderTotal = 0; // TODO: Handle order processing validation
                }

                return new OrderPurchasedResult { CustomerId = customer.Id, CustomerName = customer.Name, OrderAmount = orderTotal};
            }
            else
            {
                throw new Exception("Orders must have one or more order detail items!");
            }
        }
    }
}
