﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace IASA.Demo.Data.Generic
{
    public abstract class Repository<TDomainObject, TIdentity>
    {
        public Repository()
        {
        }

        public TDomainObject FindOne(TIdentity identity)
        {

            TDomainObject result = default(TDomainObject);

            // Execute select command
            IDataMapper<TDomainObject, TIdentity> dataFactory = this.GetDataMapper();
            DataCommand command = dataFactory.ConstructSelectCommand(identity);
            DataResult dataResult = command.GetDataResult();

            // Map result to domain model
            IObjectMapper<TDomainObject> objectMapper = this.GetObjectMapper();
            result = objectMapper.Construct(dataResult);
            return result;
        }

        public List<TDomainObject> Find(TIdentity identity)
        {
            List<TDomainObject> results = new List<TDomainObject>();

            // Execute select command
            IDataMapper<TDomainObject, TIdentity> dataFactory = this.GetDataMapper();
            DataCommand command = dataFactory.ConstructSelectCommand(identity);
            DataResult dataResult = command.GetDataResult();

            // Map result to domain model
            IObjectMapper<TDomainObject> objectMapper = this.GetObjectMapper();
            results = objectMapper.ConstructList(dataResult);
            return results;
        }

        public bool Save(TDomainObject domainObject)
        {
            //TODO: Add Support for transactions
            
            bool result = false;

            // Execute save command
            IDataMapper<TDomainObject, TIdentity> dataFactory = this.GetDataMapper();
            DataCommand command = dataFactory.ConstructSaveCommand(domainObject);
            result = command.ExecuteCommand(); 

            if (dataFactory.SaveChildren)
            {
                foreach (DataCommand childCommand in command.ChildCommands)
                {
                    result = childCommand.ExecuteCommand();
                    if (!result) // Exit save when result is false
                        break;
                }
            }

            return result;
        }


        public bool Delete(TDomainObject domainObject)
        {
            bool result = false;

            // Execute save command
            IDataMapper<TDomainObject, TIdentity> dataFactory = this.GetDataMapper();
            DataCommand command = dataFactory.ConstructDeleteCommand(domainObject);
            result = command.ExecuteCommand();

            return result;
        }

        protected abstract IDataMapper<TDomainObject, TIdentity> GetDataMapper();

        protected abstract IObjectMapper<TDomainObject> GetObjectMapper();
    }
}
